@@ -86,7 +86,7 @@ def get_fdmax(default=None):
     """
     try:
         return os.sysconf('SC_OPEN_MAX')
-    except:
+    except Exception:
         pass
     if resource is None:  # Windows
         return default
@@ -160,21 +160,17 @@ class Pidfile(object):
         (does not respond to signals)."""
         try:
             pid = self.read_pid()
-        except ValueError as exc:
+        except ValueError:
             print('Broken pidfile found. Removing it.', file=sys.stderr)
             self.remove()
             return True
         if not pid:
             self.remove()
             return True
-
-        try:
-            os.kill(pid, 0)
-        except os.error as exc:
-            if exc.errno == errno.ESRCH:
-                print('Stale pidfile exists. Removing it.', file=sys.stderr)
-                self.remove()
-                return True
+        if not pid_exists(pid):
+            print('Stale pidfile exists. Removing it.', file=sys.stderr)
+            self.remove()
+            return True
         return False
 
     def write_pid(self):
@@ -638,3 +634,32 @@ class Signals(object):
             self[signal_name] = handler
 
 signals = Signals()
+
+
+def pid_exists(pid):
+    """Check whether pid exists in the current process table.
+    UNIX only.
+    """
+    if pid < 0:
+        return False
+    if pid == 0:
+        # According to "man 2 kill" PID 0 refers to every process
+        # in the process group of the calling process.
+        # On certain systems 0 is a valid PID but we have no way
+        # to know that in a portable fashion.
+        raise ValueError('invalid PID 0')
+    try:
+        os.kill(pid, 0)
+    except OSError as err:
+        if err.errno == errno.ESRCH:
+            # ESRCH == No such process
+            return False
+        elif err.errno == errno.EPERM:
+            # EPERM clearly means there's a process to deny access to
+            return True
+        else:
+            # According to "man 2 kill" possible error values are
+            # (EINVAL, EPERM, ESRCH)
+            raise
+    else:
+        return True
