@@ -12,6 +12,8 @@ import multiprocessing
 
 import gevent
 from gevent import queue, monkey
+from gevent.lock import Semaphore
+from gevent.threadpool import ThreadPool
 monkey.patch_all(thread=False)
 
 from .. import version, json
@@ -25,17 +27,20 @@ from ..search import Search
 from .base import CommandReceiver, CommandServer, command
 from .logging import QueueHandler, ColoredStreamHandler
 try:
-    from .redis import RedisQueue
+    from .queue.redis import RedisQueue
 except ImportError:
     RedisQueue = None
-from .fqueue import FileQueue
-from .memory import MemoryQueue
+from .queue.fqueue import FileQueue
+from .queue.memory import MemoryQueue
 
+
+DEFAULT_QUEUE = MemoryQueue
 AVAILABLE_QUEUES = {
     'file': FileQueue,
-    'redis': RedisQueue or MemoryQueue,
+    'redis': RedisQueue or DEFAULT_QUEUE,
     'memory': MemoryQueue,
-    None: MemoryQueue,
+    'persistent': MemoryQueue,
+    'default': DEFAULT_QUEUE,
 }
 
 import logging
@@ -43,7 +48,10 @@ import logging
 LOG_FORMAT = "[%(asctime)s: %(levelname)s/%(processName)s:%(threadName)s] %(message)s"
 
 STOPPED = 0
+COMMIT_SLOTS = 10
 COMMIT_TIMEOUT = 1
+WRITERS_POOL_SIZE = 10
+COMMANDS_POOL_SIZE = 20
 
 WRITERS_FILE = 'Xapian-Writers.db'
 QUEUE_WORKER_MAIN = 'Xapian-Worker'
@@ -113,8 +121,8 @@ class XapiandReceiver(CommandReceiver):
         try:
             self._reopen()
             self.sendLine(">> OK")
-        except InvalidIndexError as e:
-            self.sendLine(">> ERR: Reopen: %s" % e)
+        except InvalidIndexError as exc:
+            self.sendLine(">> ERR: REOPEN: %s" % exc)
 
     @command
     def create(self, line=''):
@@ -130,8 +138,8 @@ class XapiandReceiver(CommandReceiver):
             try:
                 self._reopen(create=True, endpoints=endpoints)
                 self.active_endpoints = endpoints
-            except InvalidIndexError as e:
-                self.sendLine(">> ERR: Create: %s" % e)
+            except InvalidIndexError as exc:
+                self.sendLine(">> ERR: CREATE: %s" % exc)
             self.sendLine(">> OK")
         else:
             self.sendLine(">> ERR: [405] You must specify a valid endpoint for the database")
@@ -155,8 +163,8 @@ class XapiandReceiver(CommandReceiver):
             try:
                 self._reopen(create=False, endpoints=endpoints)
                 self.active_endpoints = endpoints
-            except InvalidIndexError as e:
-                self.sendLine(">> ERR: Using: %s" % e)
+            except InvalidIndexError as exc:
+                self.sendLine(">> ERR: OPEN: %s" % exc)
                 return
         if self.active_endpoints:
             self.sendLine(">> OK")
@@ -181,8 +189,8 @@ class XapiandReceiver(CommandReceiver):
             try:
                 self._reopen(create=True, endpoints=endpoints)
                 self.active_endpoints = endpoints
-            except InvalidIndexError as e:
-                self.sendLine(">> ERR: Using: %s" % e)
+            except InvalidIndexError as exc:
+                self.sendLine(">> ERR: USING: %s" % exc)
                 return
         if self.active_endpoints:
             self.sendLine(">> OK")
@@ -212,8 +220,8 @@ class XapiandReceiver(CommandReceiver):
                     try:
                         for result in search.results:
                             self.sendLine(json.dumps(result, ensure_ascii=False))
-                    except XapianError as e:
-                        self.sendLine(">> ERR: Unable to get results: %s" % e)
+                    except XapianError as exc:
+                        self.sendLine(">> ERR: Unable to get results: %s" % exc)
                         return
 
                     query_string = str(search.query)
@@ -224,8 +232,8 @@ class XapiandReceiver(CommandReceiver):
 
                 self.sendLine(">> OK: %s documents found in %s" % (size, format_time(time.time() - start)))
                 return size
-        except InvalidIndexError as e:
-            self.sendLine(">> ERR: %s" % e)
+        except InvalidIndexError as exc:
+            self.sendLine(">> ERR: %s" % exc)
             return
 
     @command(threaded=True, db=True, reopen=True)
@@ -289,8 +297,8 @@ class XapiandReceiver(CommandReceiver):
                 size = database.get_doccount()
                 self.sendLine(">> OK: %s documents found in %s" % (size, format_time(time.time() - start)))
                 return size
-        except InvalidIndexError as e:
-            self.sendLine(">> ERR: Count: %s" % e)
+        except InvalidIndexError as exc:
+            self.sendLine(">> ERR: COUNT: %s" % exc)
     count.__doc__ = """
     Counts matching documents.
 
@@ -427,14 +435,15 @@ class XapiandReceiver(CommandReceiver):
 
 
 class XapiandServer(CommandServer):
+    pool_size = COMMANDS_POOL_SIZE
     receiver_class = XapiandReceiver
 
     def __init__(self, *args, **kwargs):
         self.data = kwargs.pop('data', '.')
         self.databases_pool = kwargs.pop('databases_pool')
         super(XapiandServer, self).__init__(*args, **kwargs)
-        address = self.address[0] or '127.0.0.1'
-        port = self.address[1]
+        address = self.address[0] or '0.0.0.0'
+        port = self.address[1] or 8890
         self.log.info("Xapiand Server Listening to %s:%s", address, port)
 
     def buildClient(self, client_socket, address):
@@ -481,8 +490,8 @@ def _database_command(database, cmd, db, args, data='.', log=logging):
             xapian_commit(database, db, *args, data=data, log=log)
         else:
             unknown = True
-    except Exception as e:
-        log.exception("%s", e)
+    except Exception as exc:
+        log.exception("%s", exc)
         raise
     duration = time.time() - start
     docid = ' -> %s' % docid if docid else ''
@@ -504,11 +513,13 @@ def _database_commit(database, to_commit, commit_lock, timeouts, force=False, da
         do_commit = locked = force and commit_lock.acquire()  # If forcing, wait for the lock
         if not do_commit:
             do_commit = dt0 <= expires_max
+            if do_commit:
+                log.warning("Commit maximum expiration reached, commit forced! (%s)", db)
         if not do_commit:
             if dt1 <= expires_delayed or dt2 <= expires:
                 do_commit = locked = commit_lock.acquire(False)
                 if not locked:
-                    log.debug("Out of commit slots, commit delayed! (%s)", db)
+                    log.warning("Out of commit slots, commit delayed! (%s)", db)
         if do_commit:
             try:
                 _database_command(database, 'COMMIT', db, (), data=data, log=log)
@@ -558,6 +569,9 @@ def _writer_loop(databases, databases_pool, db, tq, commit_lock, timeouts, data,
     name = _database_name(db)
     to_commit = {}
 
+    tid = threading.current_thread().name.rsplit('-', 1)[-1]
+    threading.current_thread().name = '%s-%s' % (name[:14], tid)
+
     start = last = time.time()
     log.debug("New writer %s: %s", name, db)
 
@@ -648,17 +662,18 @@ def server_run(loglevel, log_queue, address, port, commit_slots, commit_timeout,
     log.addHandler(QueueHandler(log_queue))
     log.setLevel(loglevel)
 
-    commit_slots = commit_slots or multiprocessing.cpu_count()
+    if not commit_slots:
+        commit_slots = COMMIT_SLOTS
 
     if commit_timeout is None:
         commit_timeout = COMMIT_TIMEOUT
     timeout = min(max(int(round(commit_timeout * 0.3)), 1), 3)
 
-    PQueue = AVAILABLE_QUEUES.get(queue) or AVAILABLE_QUEUES[None]
+    PQueue = AVAILABLE_QUEUES.get(queue) or AVAILABLE_QUEUES['default']
     mode = "with multiple threads and %s commit slots using %s" % (commit_slots, PQueue.__name__)
-    log.warning("Starting Xapiand Server %s %s [%s] (pid:%s)", version, mode, loglevel, os.getpid())
+    log.warning("Starting Xapiand Server v%s %s [%s] (pid:%s)", version, mode, loglevel, os.getpid())
 
-    commit_lock = threading.Semaphore(commit_slots)
+    commit_lock = Semaphore(commit_slots)
     timeouts = Obj(
         timeout=timeout,
         commit=commit_timeout * 1.0,
@@ -675,26 +690,35 @@ def server_run(loglevel, log_queue, address, port, commit_slots, commit_timeout,
     gevent.signal(signal.SIGINT, xapian_server.close)
 
     log.debug("Starting server...")
-    xapian_server.start()
+    try:
+        xapian_server.start()
+    except Exception as exc:
+        log.error("Cannot start server: %s", exc)
+        sys.exit(-1)
 
     pq = get_queue(name=QUEUE_WORKER_MAIN, log=log)
 
+    pool_size = WRITERS_POOL_SIZE
+    pool_size_warning = int(pool_size / 3.0 * 2.0)
+    writers_pool = ThreadPool(pool_size)
+
     def start_writer(db):
         db = build_url(*parse_url(db.strip()))
         name = _database_name(db)
         try:
             tq = None
             t, tq = databases[db]
-            if not t.is_alive():
+            if t.ready():
                 raise KeyError
         except KeyError:
             tq = tq or get_queue(name=os.path.join(data, name), log=log)
-            t = threading.Thread(
-                target=_writer_loop,
-                name=name[:14],
-                args=(databases, databases_pool, db, tq, commit_lock, timeouts, data, log))
+            pool_used = len(writers_pool)
+            if not (pool_size_warning - pool_used) % 10:
+                log.warning("Writers pool is close to be full (%s/%s)", pool_used, pool_size)
+            elif pool_used == pool_size:
+                log.error("Writers poll is full! (%s/%s)", pool_used, pool_size)
+            t = writers_pool.spawn(_writer_loop, databases, databases_pool, db, tq, commit_lock, timeouts, data, log)
             databases[db] = (t, tq)
-            t.start()
         return db, name, t, tq
 
     if PQueue.persistent:
@@ -746,7 +770,7 @@ def server_run(loglevel, log_queue, address, port, commit_slots, commit_timeout,
     if PQueue.persistent:
         with open(writers_file, 'wt') as epfile:
             for db, (t, tq) in databases.items():
-                if t.is_alive():
+                if not t.ready():
                     epfile.write("%s\n" % db)
 
     # Wake up writers:
@@ -758,7 +782,7 @@ def server_run(loglevel, log_queue, address, port, commit_slots, commit_timeout,
 
     log.debug("Worker joining %s threads...", len(databases))
     for t, tq in databases.values():
-        t.join()
+        t.wait()
 
     xapian_cleanup(databases_pool, 0, data=data, log=log)
 
@@ -786,6 +810,7 @@ def xapiand_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask
         target=logger_run,
         args=(loglevel, log_queue, logfile, pidfile),
     )
+    logger_job.daemon = True
     logger_job.start()
 
     server_job = multiprocessing.Process(
@@ -793,6 +818,7 @@ def xapiand_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask
         target=server_run,
         args=(loglevel, log_queue, address, port, commit_slots, commit_timeout, data),
     )
+    server_job.daemon = True
     server_job.start()
 
     quit = 0
